package java.net;

import sun.net.ConnectionResetException;
import java.io.FileDescriptor;
import java.nio.channels.FileChannel;
import java.io.IOException;
import java.io.FileOutputStream;

class SocketOutputStream extends FileOutputStream
{
    private AbstractPlainSocketImpl impl;
    private byte[] temp;
    private Socket socket;
    private boolean closing;
    
    SocketOutputStream(AbstractPlainSocketImpl impl) throws IOException {
        super(impl.getFileDescriptor());
        this.impl = null;
        this.temp = new byte[1];
        this.socket = null;
        this.closing = false;
        this.impl = impl;
        this.socket = impl.getSocket();
    }
    
    @Override
    public final FileChannel getChannel() {
        return null;
    }
    
    private native void socketWrite0(FileDescriptor p0, byte[] p1, int p2, int p3) throws IOException;
    
    private void socketWrite(byte[] array, int i, int j) throws IOException {
        if (j <= 0 || i < 0 || j > array.length - i) {
            if (j == 0) {
                return;
            }
            throw new ArrayIndexOutOfBoundsException("len == " + j + " off == " + i + " buffer length == " + array.length);
        }
        else {
            FileDescriptor acquireFD = this.impl.acquireFD();
            try {
                this.socketWrite0(acquireFD, array, i, j);
            }
            catch (SocketException ex) {
                if ((ex instanceof ConnectionResetException)) {
                    this.impl.setConnectionResetPending();
                    ex = new SocketException("Connection reset");
                }
                if (this.impl.isClosedOrPending()) {
                    throw new SocketException("Socket closed");
                }
                throw ex;
            }
            finally {
                this.impl.releaseFD();
            }
            return;
        }
    }
    
    @Override
    public void write(int n) throws IOException {
        this.temp[0] = (byte)n;
        this.socketWrite(this.temp, 0, 1);
    }
    
    @Override
    public void write(byte[] array) throws IOException {
        this.socketWrite(array, 0, array.length);
    }
    
    @Override
    public void write(byte[] array, int n, int n2) throws IOException {
        this.socketWrite(array, n, n2);
    }
    
    @Override
    public void close() throws IOException {
        if (this.closing) {
            return;
        }
        this.closing = true;
        if (this.socket != null) {
            if (!this.socket.isClosed()) {
                this.socket.close();
            }
        }
        else {
            this.impl.close();
        }
        this.closing = false;
        return;
    }
    
    @Override
    protected void finalize() {
    }
    
    private static native void init();
    
    static {
        init();
    }
}
